// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
import os from "node:os";
import path from "node:path";
import fs from "node:fs";
import fsAsync from "node:fs/promises";
import toml from "toml";
import _Ajv from "ajv";
const Ajv = _Ajv;
const ajv = new Ajv();
const bindingSchema = {
    type: "object",
    nullable: true,
    properties: {
        shift: { type: "boolean", nullable: true },
        control: { type: "boolean", nullable: true },
        key: { type: "string" },
    },
    required: ["key"],
};
const promptPatternsSchema = {
    type: "array",
    nullable: true,
    items: {
        type: "object",
        properties: {
            regex: { type: "string" },
            postfix: { type: "string" },
        },
        required: ["regex", "postfix"],
    },
};
const configSchema = {
    type: "object",
    nullable: true,
    properties: {
        bindings: {
            type: "object",
            nullable: true,
            properties: {
                nextSuggestion: bindingSchema,
                previousSuggestion: bindingSchema,
                dismissSuggestions: bindingSchema,
                acceptSuggestion: bindingSchema,
            },
        },
        prompt: {
            type: "object",
            nullable: true,
            properties: {
                bash: promptPatternsSchema,
                pwsh: promptPatternsSchema,
                powershell: promptPatternsSchema,
                xonsh: promptPatternsSchema,
            },
        },
    },
    additionalProperties: false,
};
const configFile = ".inshellisenserc";
const cachePath = path.join(os.homedir(), ".inshellisense");
const configPath = path.join(os.homedir(), configFile);
let globalConfig = {
    bindings: {
        nextSuggestion: { key: "down" },
        previousSuggestion: { key: "up" },
        acceptSuggestion: { key: "tab" },
        dismissSuggestions: { key: "escape" },
    },
};
export const getConfig = () => globalConfig;
export const loadConfig = async (program) => {
    if (fs.existsSync(configPath)) {
        let config;
        try {
            config = toml.parse((await fsAsync.readFile(configPath)).toString());
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
        }
        catch (e) {
            program.error(`${configFile} is invalid toml. Parsing error on line ${e.line}, column ${e.column}: ${e.message}`);
        }
        const isValid = ajv.validate(configSchema, config);
        if (!isValid) {
            program.error(`${configFile} is invalid: ${ajv.errorsText()}`);
        }
        globalConfig = {
            bindings: {
                nextSuggestion: config?.bindings?.nextSuggestion ?? globalConfig.bindings.nextSuggestion,
                previousSuggestion: config?.bindings?.previousSuggestion ?? globalConfig.bindings.previousSuggestion,
                acceptSuggestion: config?.bindings?.acceptSuggestion ?? globalConfig.bindings.acceptSuggestion,
                dismissSuggestions: config?.bindings?.dismissSuggestions ?? globalConfig.bindings.dismissSuggestions,
            },
            prompt: {
                bash: config.prompt?.bash,
                powershell: config.prompt?.powershell,
                xonsh: config.prompt?.xonsh,
                pwsh: config.prompt?.pwsh,
            },
        };
    }
};
export const deleteCacheFolder = async () => {
    const cliConfigPath = path.join(os.homedir(), cachePath);
    if (fs.existsSync(cliConfigPath)) {
        fs.rmSync(cliConfigPath, { recursive: true });
    }
};
